@@ -618,7 +618,7 @@ class verify_files_TestCase(
             host="foo",
             check_only=None,
             check_version=None,
-            unsigned_upload=None,
+            allow_unsigned_uploads=None,
             debug=None,
             )
 
@@ -666,8 +666,10 @@ class verify_files_TestCase(
         setup_changes_file_fixtures(self)
         set_changes_file_scenario(self, 'exist-minimal')
         self.test_args.update(dict(
-                path=os.path.dirname(self.changes_file_double.path),
-                filename=os.path.basename(self.changes_file_double.path),
+                changes_file_directory=os.path.dirname(
+                    self.changes_file_double.path),
+                changes_file_name=os.path.basename(
+                    self.changes_file_double.path),
                 ))
 
         patch_os_stat(self)
@@ -905,7 +907,7 @@ class verify_files_TestCase(
                 self.expected_source_control_file_path,
                 self.runtime_config_parser,
                 self.test_args['check_only'],
-                self.test_args['unsigned_upload'],
+                self.test_args['allow_unsigned_uploads'],
                 unittest.mock.ANY,
                 self.test_args['debug'])
 
@@ -1172,8 +1174,10 @@ class guess_upload_host_TestCase(
     def set_test_args(self):
         """ Set the arguments for the test call to the function. """
         self.test_args = dict(
-                path=os.path.dirname(self.changes_file_double.path),
-                filename=os.path.basename(self.changes_file_double.path),
+                changes_file_directory=os.path.dirname(
+                    self.changes_file_double.path),
+                changes_file_name=os.path.basename(
+                    self.changes_file_double.path),
                 config=self.runtime_config_parser,
                 )
 
@@ -1222,7 +1226,7 @@ class guess_upload_host_TestCase(
         self.assertIn(expected_output, sys.stdout.getvalue())
 
 
-# Copyright © 2015–2019 Ben Finney <bignose@debian.org>
+# Copyright © 2015–2021 Ben Finney <bignose@debian.org>
 #
 # This is free software: you may copy, modify, and/or distribute this work
 # under the terms of the GNU General Public License as published by the
