@@ -14,6 +14,7 @@ import getpass
 import http.client
 import importlib
 import io
+import itertools
 import os
 import os.path
 import pkgutil
@@ -23,11 +24,11 @@ import sys
 import tempfile
 import textwrap
 import unittest.mock
+import urllib.parse
+import urllib.request
 
-import httpretty
 import testscenarios
 import testtools
-import urllib.parse
 
 import dput.dput
 import dput.helper.dputhelper
@@ -853,6 +854,58 @@ class ftp_upload_ProgressTestCase(ftp_upload_TestCase):
                     testtools.matchers.Contains(expected_output))
 
 
+def make_mock_httprequest(
+        *,
+        method="GET",
+        url=None,
+        header_fields=None,
+        body=None):
+    """ Make an instrumented mock for `urllib.request.Request`. """
+    request = unittest.mock.Mock(spec=urllib.request.Request)
+    request.url = url
+    request.headers = header_fields
+    if body is None:
+        body = ""
+    body_data = body.encode(dput.methods.http.MESSAGE_BODY_ENCODING)
+    request.data = body_data
+
+    return request
+
+
+reason_by_http_status = {
+    http.client.OK: "Okay",
+    http.client.NON_AUTHORITATIVE_INFORMATION: "Non-Authoritative Information",
+    http.client.INTERNAL_SERVER_ERROR: "Internal Server Error",
+    http.client.UNAUTHORIZED: "Unauthorized",
+}
+
+
+def make_mock_httpresponse(
+        *,
+        version="HTTP/1.1",
+        status=http.client.OK,
+        header_fields=None,
+        body=None):
+    """ Make an instrumented mock for `http.client.HTTPResponse`. """
+    response = unittest.mock.Mock(spec=http.client.HTTPResponse)
+    response.version = version
+    response.status = status
+    response.reason = reason_by_http_status[status]
+
+    if header_fields is None:
+        header_fields = {}
+    response.headers = {name: value for (name, value) in header_fields.items()}
+    if body is None:
+        body = ""
+    body_data = body.encode(dput.methods.http.MESSAGE_BODY_ENCODING)
+    response.msg = body_data
+    response.read.return_value = body_data
+    response.getheader.side_effect = (lambda n: header_fields[n])
+    response.getheaders.return_value = list(header_fields.items())
+
+    return response
+
+
 class http_upload_TestCase(upload_TestCase):
     """ Base for test cases for `methods.http.upload` function. """
 
@@ -881,11 +934,8 @@ class http_upload_TestCase(upload_TestCase):
         """ Set up test fixtures. """
         super().setUp()
 
-        httpretty.enable()
-        self.addCleanup(httpretty.reset)
-
         self.set_response_header_fields()
-        self.patch_put_requests()
+        self.patch_connection()
 
         patch_getpass_getpass(self)
         self.fake_password = self.getUniqueString()
@@ -922,14 +972,37 @@ class http_upload_TestCase(upload_TestCase):
         if not hasattr(self, 'response_header_fields'):
             self.response_header_fields = {}
 
-    def patch_put_requests(self):
-        """ Patch the HTTP PUT requests. """
-        self.path_by_request_uri = {}
-        for path in self.paths_to_upload:
-            upload_uri = self.make_upload_uri(os.path.basename(path))
-            self.path_by_request_uri[upload_uri] = path
-            httpretty.register_uri(
-                    httpretty.PUT, upload_uri, responses=self.responses)
+    def patch_connection(self):
+        """ Patch the connection instance. """
+        mock_connection = unittest.mock.Mock(spec=http.client.HTTPConnection)
+        mock_connection.getresponse.side_effect = itertools.chain(
+                self.responses,
+                itertools.repeat(self.responses[-1]))
+        def putrequest(method, url, *, skip_accept_encoding=True):
+            request = make_mock_httprequest(
+                method=method,
+                header_fields={},
+                url=url)
+            mock_connection.latest_request = request
+        mock_connection.putrequest = unittest.mock.Mock(
+            spec=http.client.HTTPConnection.putrequest,
+            wraps=putrequest)
+        def putheader(header, *values):
+            values = list(values)
+            mock_connection.latest_request.headers[header] = "\r\n".join(values)
+        mock_connection.putheader = unittest.mock.Mock(
+            spec=http.client.HTTPConnection.putheader,
+            wraps=putheader)
+        def send(data):
+            mock_connection.latest_request.data = data
+        mock_connection.send = unittest.mock.Mock(
+            spec=http.client.HTTPConnection.send,
+            wraps=send)
+        self.mock_connection = mock_connection
+        self.mock_connection_class = unittest.mock.Mock(
+                spec=http.client.HTTPConnection,
+                return_value=self.mock_connection)
+        self.test_args['connection_class'] = self.mock_connection_class
 
 
 class http_upload_SuccessTestCase(http_upload_TestCase):
@@ -938,20 +1011,22 @@ class http_upload_SuccessTestCase(http_upload_TestCase):
     response_scenarios = [
             ('okay', {
                 'responses': [
-                    httpretty.Response(
+                    make_mock_httpresponse(
                         status=http.client.OK,
                         body=textwrap.dedent("""\
                             Lorem ipsum, dolor sit amet.
-                            """)),
+                            """),
+                        ),
                     ],
                 }),
             ('chatter', {
                 'responses': [
-                    httpretty.Response(
+                    make_mock_httpresponse(
                         status=http.client.NON_AUTHORITATIVE_INFORMATION,
                         body=textwrap.dedent("""\
                             Lorem ipsum, dolor sit amet.
-                            """)),
+                            """),
+                        ),
                     ],
                 }),
             ]
@@ -1005,7 +1080,7 @@ class http_upload_SuccessTestCase(http_upload_TestCase):
         registry = FileDouble.get_registry_for_testcase(self)
         path = self.paths_to_upload[-1]
         double = registry[path]
-        request = httpretty.last_request()
+        request = self.mock_connection.latest_request
         expected_fields = {
                 'User-Agent': "dput",
                 'Connection': "close",
@@ -1029,10 +1104,12 @@ class http_upload_ProgressTestCase(http_upload_TestCase):
     response_scenarios = [
             ('okay', {
                 'responses': [
-                    httpretty.Response(
-                        status=http.client.OK, body=textwrap.dedent("""\
-                        Lorem ipsum, dolor sit amet.
-                        """)),
+                    make_mock_httpresponse(
+                        status=http.client.OK,
+                        body=textwrap.dedent("""\
+                            Lorem ipsum, dolor sit amet.
+                            """),
+                        ),
                     ],
                 }),
             ]
@@ -1172,13 +1249,28 @@ class http_upload_ResponseErrorTestCase(http_upload_TestCase):
             if scenario['paths_to_upload'])
 
     response_scenarios = [
-            ('server-error', {
+            ('server-error body-text', {
                 'responses': [
-                    httpretty.Response(
+                    make_mock_httpresponse(
                         status=http.client.INTERNAL_SERVER_ERROR,
                         body=textwrap.dedent("""\
                             Lorem ipsum, dolor sit amet.
-                            """)),
+                            """),
+                        ),
+                    ],
+                'expected_final_status': (
+                    http.client.INTERNAL_SERVER_ERROR,
+                    "Internal Server Error"),
+                'expected_debug_output': textwrap.dedent("""\
+                    Lorem ipsum, dolor sit amet.
+                    """),
+                'expected_exit_status': EXIT_STATUS_FAILURE,
+                }),
+            ('server-error body-empty', {
+                'responses': [
+                    make_mock_httpresponse(
+                        status=http.client.INTERNAL_SERVER_ERROR,
+                        body=""),
                     ],
                 'expected_final_status': (
                     http.client.INTERNAL_SERVER_ERROR,
@@ -1187,21 +1279,30 @@ class http_upload_ResponseErrorTestCase(http_upload_TestCase):
                 }),
             ('authorization-rejected', {
                 'responses': [
-                    httpretty.Response(
+                    make_mock_httpresponse(
                         status=http.client.UNAUTHORIZED,
-                        www_authenticate="Basic realm=\"pellentesque\"",
+                        header_fields={
+                            'WWW-Authenticate': "Basic realm=\"pellentesque\"",
+                            },
                         body=textwrap.dedent("""\
                             Lorem ipsum, dolor sit amet.
-                            """)),
-                    httpretty.Response(
+                            """),
+                        ),
+                    make_mock_httpresponse(
                         status=http.client.UNAUTHORIZED,
-                        www_authenticate="Basic realm=\"pellentesque\"",
+                        header_fields={
+                            'WWW-Authenticate': "Basic realm=\"pellentesque\"",
+                            },
                         body=textwrap.dedent("""\
                             Lorem ipsum, dolor sit amet.
-                            """)),
+                            """),
+                        ),
                     ],
                 'expected_final_status': (
                     http.client.UNAUTHORIZED, "Unauthorized"),
+                'expected_debug_output': textwrap.dedent("""\
+                    Lorem ipsum, dolor sit amet.
+                    """),
                 'expected_exit_status': EXIT_STATUS_FAILURE,
                 }),
             ]
@@ -1227,6 +1328,34 @@ class http_upload_ResponseErrorTestCase(http_upload_TestCase):
                 """).format(status=code, reason=reason)
         self.assertIn(expected_output, sys.stdout.getvalue())
 
+    def test_emits_error_response_body_text_when_debug_true(self):
+        """ Should emit error response body text when debug is true. """
+        self.test_args['debug'] = True
+        if not hasattr(self, 'expected_debug_output'):
+            self.skipTest("no response body expected")
+        if self.expected_final_status[0] == http.client.UNAUTHORIZED:
+            self.skipTest("FIXME: can't see how this ever worked?")
+        try:
+            self.function_to_test(**self.test_args)
+        except FakeSystemExit:
+            pass
+        expected_output = self.expected_debug_output
+        self.assertIn(expected_output, sys.stdout.getvalue())
+
+    def test_emits_no_response_body_text_when_debug_false(self):
+        """ Should emit no response body text when ‘debug’ is false. """
+        self.test_args['debug'] = False
+        if not hasattr(self, 'expected_debug_output'):
+            self.skipTest("no response body expected")
+        if self.expected_final_status[0] == http.client.UNAUTHORIZED:
+            self.skipTest("FIXME: can't see how this ever worked?")
+        try:
+            self.function_to_test(**self.test_args)
+        except FakeSystemExit:
+            pass
+        unwanted_output = self.expected_debug_output
+        self.assertNotIn(unwanted_output, sys.stdout.getvalue())
+
     def test_calls_sys_exit_when_response_status_error(self):
         """ Should call `sys.exit` when response status is error. """
         if self.expected_final_status[0] == http.client.UNAUTHORIZED:
@@ -1247,17 +1376,21 @@ class http_upload_ResponseUnauthorizedTestCase(http_upload_TestCase):
     response_scenarios = [
             ('authorization-accepted', {
                 'responses': [
-                    httpretty.Response(
+                    make_mock_httpresponse(
                         status=http.client.UNAUTHORIZED,
-                        www_authenticate="Basic realm=\"pellentesque\"",
+                        header_fields={
+                            'WWW-Authenticate': "Basic realm=\"pellentesque\"",
+                            },
                         body=textwrap.dedent("""\
                             Lorem ipsum, dolor sit amet.
-                            """)),
-                    httpretty.Response(
+                            """),
+                        ),
+                    make_mock_httpresponse(
                         status=http.client.OK,
                         body=textwrap.dedent("""\
                             Lorem ipsum, dolor sit amet.
-                            """)),
+                            """),
+                        ),
                     ],
                 'expected_final_status': (http.client.OK, "Okay"),
                 'expected_exit_status': EXIT_STATUS_SUCCESS,
@@ -1550,6 +1683,7 @@ class rsync_upload_TestCase(ssh_channel_upload_TestCase):
         """ Set up test fixtures. """
         super().setUp()
 
+        self.expected_rsync_options = dput.methods.rsync.rsync_options
         self.set_rsync_subprocess_double()
 
         self.expected_ssh_options = []
@@ -1572,9 +1706,7 @@ class rsync_upload_TestCase(ssh_channel_upload_TestCase):
         command_file_path = "/usr/bin/rsync"
         argv = [os.path.basename(command_file_path)]
         argv.extend(self.paths_to_upload)
-        argv.extend([
-                "--copy-links", "--progress", "--partial",
-                "-zave", "ssh -x"])
+        argv.extend(self.expected_rsync_options)
         argv.append(make_remote_spec(
                 username=self.login, host=self.test_args['fqdn'],
                 dir_path=self.incoming_path))
@@ -1660,7 +1792,7 @@ class rsync_upload_TestCase(ssh_channel_upload_TestCase):
         sys.exit.assert_called_with(EXIT_STATUS_FAILURE)
 
 
-# Copyright © 2015–2019 Ben Finney <bignose@debian.org>
+# Copyright © 2015–2021 Ben Finney <bignose@debian.org>
 #
 # This is free software: you may copy, modify, and/or distribute this work
 # under the terms of the GNU General Public License as published by the
